@@ -808,11 +808,24 @@ handle_pgres_error(ConnectionClass *self, const PGresult *pgres,
 	char	   *errseverity;
 	char	   *errseverity_nonloc = NULL;
 	char	   *errprimary = NULL;
+
+	char	   *errdetail = NULL;
+	char	   *errhint = NULL;
+	char	   *errstatementposition = NULL;
+	char	   *errinternalquery = NULL;
+	char	   *errcontext = NULL;
+	char	   *errschemaname = NULL;
+	char	   *errtablename = NULL;
+	char	   *errcolumnname = NULL;
+	char	   *errdatatypename = NULL;
+
 	char	   *errmsg = NULL;
-	size_t		errmsglen;
 	char	*sqlstate = NULL;
 	int	level = MIN_LOG_LEVEL;
 
+	int	display_error_level = (self->connInfo).optional_errors ? 3 : 1;
+	PQExpBufferData	errbuf = {0};
+
 	MYLOG(DETAIL_LOG_LEVEL, "entering\n");
 
 	sqlstate = PQresultErrorField(pgres, PG_DIAG_SQLSTATE);
@@ -877,12 +890,47 @@ handle_pgres_error(ConnectionClass *self, const PGresult *pgres,
 		if (errprimary == NULL)
 			errprimary = "no error information";
 	}
+	else if (display_error_level > 0)
+	{
+		errdetail = PQresultErrorField(pgres, PG_DIAG_MESSAGE_DETAIL);
+		errhint = PQresultErrorField(pgres, PG_DIAG_MESSAGE_HINT);
+		errstatementposition = PQresultErrorField(pgres, PG_DIAG_STATEMENT_POSITION);
+		errinternalquery = PQresultErrorField(pgres, PG_DIAG_INTERNAL_POSITION);
+		errcontext = PQresultErrorField(pgres, PG_DIAG_CONTEXT);
+		errschemaname = PQresultErrorField(pgres, PG_DIAG_SCHEMA_NAME);
+		errtablename = PQresultErrorField(pgres, PG_DIAG_TABLE_NAME);
+		errcolumnname = PQresultErrorField(pgres, PG_DIAG_COLUMN_NAME);
+		errdatatypename = PQresultErrorField(pgres, PG_DIAG_DATATYPE_NAME);
+	}
+	initPQExpBuffer(&errbuf);
 	if (errseverity && errprimary)
 	{
-		errmsglen = strlen(errseverity) + 2 + strlen(errprimary) + 1;
-		errmsg = malloc(errmsglen);
-		if (errmsg)
-			snprintf(errmsg, errmsglen, "%s: %s", errseverity, errprimary);
+		printfPQExpBuffer(&errbuf, "%s: %s", errseverity, errprimary);
+		if (display_error_level > 0 && errdetail)
+			appendPQExpBuffer(&errbuf, "\nDETAIL: %s", errdetail);
+		if (display_error_level > 1)
+		{
+			if (errhint)
+				appendPQExpBuffer(&errbuf, "\nHINT: %s", errhint);
+		}
+		if (display_error_level > 2)
+		{
+			if (errstatementposition)
+				appendPQExpBuffer(&errbuf, "\nSTATEMENT_POSITION: %s", errstatementposition);
+			if (errinternalquery)
+				appendPQExpBuffer(&errbuf, "\nINTERNAL_QUERY: %s", errinternalquery);
+			if (errcontext)
+				appendPQExpBuffer(&errbuf, "\nCONTEXT: %s", errcontext);
+			if (errschemaname)
+				appendPQExpBuffer(&errbuf, "\nSCHEMA_NAME: %s", errschemaname);
+			if (errtablename)
+				appendPQExpBuffer(&errbuf, "\nTABLE_NAME: %s", errtablename);
+			if (errcolumnname)
+				appendPQExpBuffer(&errbuf, "\nCOLUMN_NAME: %s", errcolumnname);
+			if (errdatatypename)
+				appendPQExpBuffer(&errbuf, "\nDATATYPE_NAME :%s", errdatatypename);
+		}
+		errmsg = errbuf.data;
 	}
 	if (errmsg == NULL)
 		errmsg = errprimary;
@@ -931,22 +979,11 @@ handle_pgres_error(ConnectionClass *self, const PGresult *pgres,
 	}
 
 cleanup:
-	if (errmsg != errprimary)
-		free(errmsg);
+	if (!PQExpBufferDataBroken(errbuf))
+		termPQExpBuffer(&errbuf);
 	LIBPQ_update_transaction_status(self);
 }
 
-/*
- * This is a libpq notice receiver callback, for handling incoming NOTICE
- * messages while processing a query.
- */
-typedef struct
-{
-	ConnectionClass *conn;
-	const char *comment;
-	QResultClass *res;
-} notice_receiver_arg;
-
 void
 receive_libpq_notice(void *arg, const PGresult *pgres)
 {
@@ -954,6 +991,8 @@ receive_libpq_notice(void *arg, const PGresult *pgres)
 	{
 		notice_receiver_arg *nrarg = (notice_receiver_arg *) arg;
 
+		if (NULL != nrarg->stmt)
+			nrarg->stmt->has_notice = 1;
 		handle_pgres_error(nrarg->conn, pgres, nrarg->comment, nrarg->res, FALSE);
 	}
 }
@@ -1313,7 +1352,7 @@ static int handle_show_results(const QResultClass *res)
 	const QResultClass	*qres;
 	ConnectionClass		*conn = QR_get_conn(res);
 
-	for (qres = res; qres; qres = qres->next)
+	for (qres = res; qres; qres = QR_nextr(qres))
 	{
 		if (!qres->command ||
 		    stricmp(qres->command, "SHOW") != 0)
@@ -1727,10 +1766,11 @@ CC_internal_rollback(ConnectionClass *self, int rollback_type, BOOL ignore_abort
  * * Send appendq, read result.
  *
  */
-QResultClass *
+QResultHold
 CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UDWORD flag, StatementClass *stmt, const char *appendq)
 {
 	CSTR	func = "CC_send_query";
+	QResultHold	rhold = {0};
 	QResultClass *cmdres = NULL,
 			   *retres = NULL,
 			   *res = NULL;
@@ -1776,7 +1816,7 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 		appendPQExpBuffer(&pbuf, "The connection is down\nFailed to send '%s'", query);
 		CC_set_error(self, CONNECTION_COULD_NOT_SEND, pbuf.data, func);
 		termPQExpBuffer(&pbuf);
-		return NULL;
+		return rhold;
 	}
 
 	ENTER_INNER_CONN_CS(self, func_cs_count);
@@ -1784,7 +1824,7 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 	if ((NULL == query) || (query[0] == '\0'))
 	{
 		CLEANUP_FUNC_CONN_CS(func_cs_count, self);
-		return NULL;
+		return rhold;
 	}
 
 	/*
@@ -1794,20 +1834,22 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 	 */
 	if (appendq && ignore_roundtrip_time)
 	{
-		res = CC_send_query_append(self, query, qi, flag, stmt, NULL);
+		QResultHold rholda;
+
+		rhold = CC_send_query_append(self, query, qi, flag, stmt, NULL);
 		if (QR_command_maybe_successful(res))
 		{
-			cmdres = CC_send_query_append(self, appendq, qi, flag & (~(GO_INTO_TRANSACTION)), stmt, NULL);
-			if (QR_command_maybe_successful(cmdres))
-				res->next = cmdres;
+			rholda = CC_send_query_append(self, appendq, qi, flag & (~(GO_INTO_TRANSACTION)), stmt, NULL);
+			if (QR_command_maybe_successful(rholda.first))
+				QR_concat(rhold.last, rholda.first);
 			else
 			{
-				QR_Destructor(res);
-				res = cmdres;
+				QR_Destructor(rhold.first);
+				rhold = rholda;
 			}
 		}
 		CLEANUP_FUNC_CONN_CS(func_cs_count, self);
-		return res;
+		return rhold;
 	}
 
 	rollback_on_error = (flag & ROLLBACK_ON_ERROR) != 0;
@@ -1887,6 +1929,7 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 	nrarg.conn = self;
 	nrarg.comment = func;
 	nrarg.res = NULL;
+	nrarg.stmt = stmt;
 	PQsetNoticeReceiver(self->pqconn, receive_libpq_notice, &nrarg);
 
 	QLOG(0, "PQsendQuery: %p '%s'\n", self->pqconn, query_buf.data);
@@ -1935,15 +1978,15 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 
 				if (query_completed)	/* allow for "show" style notices */
 				{
-					res->next = QR_Constructor();
-					if (!res->next)
+					QR_concat(res, QR_Constructor());
+					if (!QR_nextr(res))
 					{
 						CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, "Could not create result info in send_query.", func);
 						ReadyToReturn = TRUE;
 						retres = NULL;
 						break;
 					}
-					res = res->next;
+					res = QR_nextr(res);
 					nrarg.res = res;
 				}
 
@@ -2040,6 +2083,8 @@ MYLOG(DETAIL_LOG_LEVEL, "Discarded a RELEASE result\n");
 				break;
 			case PGRES_NONFATAL_ERROR:
 				handle_pgres_error(self, pgres, "send_query", res, FALSE);
+				if (stmt)
+					stmt->has_notice = 1;
 				break;
 
 			case PGRES_BAD_RESPONSE:
@@ -2056,8 +2101,8 @@ MYLOG(DETAIL_LOG_LEVEL, "Discarded a RELEASE result\n");
 			case PGRES_SINGLE_TUPLE:
 				if (query_completed)
 				{
-					res->next = QR_Constructor();
-					if (!res->next)
+					QR_concat(res, QR_Constructor());
+					if (!QR_nextr(res))
 					{
 						CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, "Could not create result info in send_query.", func);
 						ReadyToReturn = TRUE;
@@ -2066,16 +2111,16 @@ MYLOG(DETAIL_LOG_LEVEL, "Discarded a RELEASE result\n");
 					}
 					if (create_keyset)
 					{
-						QR_set_haskeyset(res->next);
+						QR_set_haskeyset(QR_nextr(res));
 						if (stmt)
 						{
 							if (stmt->num_key_fields < 0) /* for safety */
 								CheckPgClassInfo(stmt);
-							res->next->num_key_fields = stmt->num_key_fields;
+							QR_nextr(res)->num_key_fields = stmt->num_key_fields;
 						}
 					}
-					MYLOG(0, " 'T' no result_in: res = %p\n", res->next);
-					res = res->next;
+					MYLOG(0, " 'T' no result_in: res = %p\n", QR_nextr(res));
+					res = QR_nextr(res);
 					nrarg.res = res;
 
 					if (qi)
@@ -2149,15 +2194,15 @@ MYLOG(DETAIL_LOG_LEVEL, "Discarded a RELEASE result\n");
 			case PGRES_COPY_IN:
 				if (query_completed)
 				{
-					res->next = QR_Constructor();
-					if (!res->next)
+					QR_concat(res, QR_Constructor());
+					if (!QR_nextr(res))
 					{
 						CC_set_error(self, CONNECTION_COULD_NOT_RECEIVE, "Could not create result info in send_query.", func);
 						ReadyToReturn = TRUE;
 						retres = NULL;
 						break;
 					}
-					res = res->next;
+					res = QR_nextr(res);
 					nrarg.res = res;
 				}
 				QR_set_rstatus(res, PORES_COPY_IN);
@@ -2254,12 +2299,12 @@ MYLOG(DETAIL_LOG_LEVEL, " rollback_on_error=%d CC_is_in_trans=%d discard_next_sa
 				 *	discard results other than errors.
 				 */
 				QResultClass	*qres;
-				for (qres = retres; qres->next; qres = retres)
+				for (qres = retres; QR_nextr(qres); qres = retres)
 				{
 					if (QR_get_aborted(qres))
 						break;
-					retres = qres->next;
-					qres->next = NULL;
+					retres = QR_nextr(qres);
+					QR_detach(qres);
 					QR_Destructor(qres);
 				}
 				/*
@@ -2295,7 +2340,9 @@ MYLOG(DETAIL_LOG_LEVEL, " ignored abort_on_conn\n");
 
 	if (retres)
 		QR_set_conn(retres, self);
-	return retres;
+	rhold.first = retres;
+	rhold.last = res;
+	return rhold;
 }
 
 #define MAX_SEND_FUNC_ARGS	3
@@ -2653,9 +2700,10 @@ LIBPQ_update_transaction_status(ConnectionClass *self)
 	if (!self->pqconn)
 		return;
 
+	MYLOG(DETAIL_LOG_LEVEL, "transactionStatus=%d\n", PQtransactionStatus(self->pqconn));
 	switch (PQtransactionStatus(self->pqconn))
 	{
-		case PQTRANS_IDLE:
+		case PQTRANS_IDLE:	// correspond to 'I' in ReadyForQuery message
 			if (CC_is_in_trans(self))
 			{
 				if (CC_is_in_error_trans(self))
@@ -2665,7 +2713,7 @@ LIBPQ_update_transaction_status(ConnectionClass *self)
 			}
 			break;
 
-		case PQTRANS_INTRANS:
+		case PQTRANS_INTRANS:	// correspond to 'T' in ReadyForQuery message
 			CC_set_in_trans(self);
 			if (CC_is_in_error_trans(self))
 			{
@@ -2674,17 +2722,16 @@ LIBPQ_update_transaction_status(ConnectionClass *self)
 			}
 			break;
 
-		case PQTRANS_INERROR:
+		case PQTRANS_INERROR:	// correspond to 'E' in ReadyForQuery message
 			CC_set_in_trans(self);
 			CC_set_in_error_trans(self);
 			break;
 
-		case PQTRANS_ACTIVE:
+		case PQTRANS_ACTIVE:	// no correspondence in ReadyForQuery message
 			/*
-			 * A query is still executing. It might have already aborted,
-			 * but all we know for sure is that we're in a transaction.
+			 * A query is still executing. It's not necessary to consider this status.
 			 */
-			CC_set_in_trans(self);
+			// CC_set_in_trans(self);
 			break;
 
 		default: 			/* unknown status */
@@ -3050,7 +3097,7 @@ my_appendPQExpBuffer(PQExpBufferData *buf, const char *fmt, const char *s, ssize
 
 /*
  *	my_appendPQExpBuffer1 is a extension of my_appendPQExpBuffer.
- *	It can have 1 more parameter than my_aapendPQExpBuffer.
+ *	It can have 1 more parameter than my_appendPQExpBuffer.
  */
 static void
 my_appendPQExpBuffer1(PQExpBufferData *buf, const char *fmt, const char *s1, const char *s)
@@ -3536,7 +3583,7 @@ CC_set_transact(ConnectionClass *self, UInt4 isolation)
 	if (self->default_isolation == 0)
 		bShow = TRUE;
 	if (bShow)
-		res = CC_send_query_append(self, ISOLATION_SHOW_QUERY, NULL, READ_ONLY_QUERY, NULL, query);
+		res = CC_send_query_append(self, ISOLATION_SHOW_QUERY, NULL, READ_ONLY_QUERY, NULL, query).first;
 	else
 		res = CC_send_query(self, query, NULL, READ_ONLY_QUERY, NULL);
 	if (!QR_command_maybe_successful(res))
