@@ -314,6 +314,8 @@ enum {
 	,SQL_ATTR_PGOPT_MAXLONGVARCHARSIZE = 65547
 	,SQL_ATTR_PGOPT_WCSDEBUG = 65548
 	,SQL_ATTR_PGOPT_MSJET = 65549
+	,SQL_ATTR_PGOPT_BATCHSIZE = 65550
+	,SQL_ATTR_PGOPT_IGNORETIMEOUT = 65551
 };
 RETCODE SQL_API PGAPI_SetConnectAttr(HDBC ConnectionHandle,
 			SQLINTEGER Attribute, PTR Value,
