@@ -618,6 +618,11 @@ pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod
 				return sqltype;
 #endif /* PG_INTERVAL_AS_SQL_INTERVAL */
 			return ansi_to_wtype(conn, SQL_VARCHAR);
+		case PG_TYPE_BIT:
+			if (1 == atttypmod)
+				return SQL_BIT;
+			else
+				return SQL_VARCHAR;
 
 		default:
 
@@ -840,6 +845,14 @@ MYLOG(DETAIL_LOG_LEVEL, "pgtype_to_name int4\n");
 			return "cidr";
 		case PG_TYPE_UUID:
 			return "uuid";
+		case PG_TYPE_VOID:
+			return "void";
+		case PG_TYPE_INT2VECTOR:
+			return "int2vector";
+		case PG_TYPE_OIDVECTOR:
+			return "oidvector";
+		case PG_TYPE_ANY:
+			return "any";
 		case PG_TYPE_INTERVAL:
 			get_interval_type(atttypmod, &tname);
 			return tname;
