@@ -432,6 +432,10 @@ typedef struct IPDFields_ IPDFields;
 typedef struct col_info COL_INFO;
 typedef struct lo_arg LO_ARG;
 
+typedef struct QResultHold_struct {
+	QResultClass *first;
+	QResultClass *last;
+} QResultHold;
 
 /*	pgNAME type define */
 typedef struct
@@ -636,11 +640,15 @@ typedef struct
 	signed char	accessible_only;
 	signed char	ignore_round_trip_time;
 	signed char	disable_keepalive;
+	signed char	disable_convert_func;
 	signed char	wcs_debug;
 	signed char	numeric_as;
+	signed char	optional_errors;
+	signed char	ignore_timeout;
 	UInt4		extra_opts;
 	Int4		keepalive_idle;
 	Int4		keepalive_interval;
+	Int4		batch_size;
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	signed char	xa_opt;
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
