@@ -378,8 +378,9 @@ SC_Constructor(ConnectionClass *conn)
 	{
 		rv->hdbc = conn;
 		rv->phstmt = NULL;
-		rv->result = NULL;
+		rv->rhold.first = rv->rhold.last = NULL;
 		rv->curres = NULL;
+		rv->parsed = NULL;
 		rv->catalog_result = FALSE;
 		rv->prepare = NON_PREPARE_STATEMENT;
 		rv->prepared = NOT_YET_PREPARED;
@@ -418,7 +419,6 @@ SC_Constructor(ConnectionClass *conn)
 		rv->put_data = FALSE;
 		rv->ref_CC_error = FALSE;
 		rv->join_info = 0;
-		rv->curr_param_result = 0;
 		SC_init_parse_method(rv);
 
 		rv->lobj_fd = -1;
@@ -528,24 +528,51 @@ SC_Destructor(StatementClass *self)
 void
 SC_init_Result(StatementClass *self)
 {
-	self->result = self->curres = NULL;
-	self->curr_param_result = 0;
+	self->rhold.first = self->rhold.last = NULL;
+	self->curres = NULL;
 	MYLOG(0, "leaving(%p)\n", self);
 }
 
 void
-SC_set_Result(StatementClass *self, QResultClass *res)
+SC_set_Result(StatementClass *self, QResultClass *first)
 {
-	if (res != self->result)
+	if (first != self->rhold.first)
 	{
-		MYLOG(0, "(%p, %p)\n", self, res);
-		QR_Destructor(self->result);
-		self->result = self->curres = res;
-		if (NULL != res)
-			self->curr_param_result = 1;
+		QResultClass *last = NULL, *res;
+
+		MYLOG(0, "(%p, %p)\n", self, first);
+		QR_Destructor(self->parsed);
+		self->parsed = NULL;
+		QR_Destructor(self->rhold.first);
+		for (res = first; res; res = QR_nextr(res))
+			last = res;
+		self->curres = first;
+		self->rhold.first = first;
+		self->rhold.last = last;
 	}
 }
 
+void
+SC_set_ResultHold(StatementClass *self, QResultHold rhold)
+{
+	if (rhold.first != self->rhold.first)
+	{
+		MYLOG(0, "(%p, {%p, %p})\n", self, rhold.first, rhold.last);
+		QR_Destructor(self->parsed);
+		self->parsed = NULL;
+		QR_Destructor(self->rhold.first);
+		self->curres = rhold.first;
+		self->rhold = rhold;
+	}
+	else if (rhold.last != self->rhold.last)
+	{
+		self->rhold.last = rhold.last;
+		if (QR_nextr(rhold.last) != NULL)
+			SC_set_error(self, STMT_INTERNAL_ERROR, "last Result is not the last result", __FUNCTION__);
+	}
+
+}
+
 /*
  *	Free parameters and free the memory from the
  *	data-at-execution parameters that was allocated in SQLPutData.
@@ -808,7 +835,6 @@ SC_reset_result_for_rerun(StatementClass *self)
 	else
 	{
 		QR_reset_for_re_execute(res);
-		self->curr_param_result = 1;
 		SC_set_Curres(self, NULL);
 	}
 }
@@ -873,6 +899,8 @@ MYLOG(DETAIL_LOG_LEVEL, "SC_clear_parse_status\n");
 	/* Free any cursors */
 	if (SC_get_Result(self))
 		SC_set_Result(self, NULL);
+	QR_Destructor(self->parsed);
+	self->parsed = NULL;
 	self->miscinfo = 0;
 	self->execinfo = 0;
 	/* self->rbonerr = 0; Never clear the bits here */
@@ -1106,7 +1134,7 @@ SC_describe(StatementClass *self)
 	QResultClass	*res;
 	MYLOG(0, "entering status = %d\n", self->status);
 
-	res = SC_get_Curres(self);
+	res = SC_get_Parsed(self);
 	if (NULL != res)
 	{
 		num_fields = QR_NumResultCols(res);
@@ -1140,7 +1168,7 @@ MYLOG(0, "              preprocess: status = READY\n");
 				self->status = STMT_DESCRIBED;
 				break;
 		}
-		if (res = SC_get_Curres(self), NULL != res)
+		if (res = SC_get_Parsed(self), NULL != res)
 		{
 			num_fields = QR_NumResultCols(res);
 			return num_fields;
@@ -1273,6 +1301,8 @@ SC_create_errorinfo(const StatementClass *self, PG_ErrorInfo *pgerror_fail_safe)
 	char		*ermsg = NULL, *sqlstate = NULL;
 	PG_ErrorInfo	*pgerror;
 
+	if (!res)
+		res = SC_get_Parsed(self);
 	if (self->pgerror)
 		return self->pgerror;
 	errornum = self->__error_number;
@@ -1281,7 +1311,7 @@ SC_create_errorinfo(const StatementClass *self, PG_ErrorInfo *pgerror_fail_safe)
 
 	looponce = (SC_get_Result(self) != res);
 	msg[0] = '\0';
-	for (loopend = FALSE; (NULL != res) && !loopend; res = res->next)
+	for (loopend = FALSE; (NULL != res) && !loopend; res = QR_nextr(res))
 	{
 		if (looponce)
 			loopend = TRUE;
@@ -1842,7 +1872,6 @@ SC_execute(StatementClass *self)
 	ConnectionClass *conn;
 	IPDFields	*ipdopts;
 	char		was_ok, was_nonfatal;
-	QResultClass	*res = NULL;
 	Int2		oldstatus,
 				numcols;
 	QueryInfo	qi;
@@ -1855,6 +1884,7 @@ SC_execute(StatementClass *self)
 	int		errnum_sav = STMT_OK, errnum;
 	char		*errmsg_sav = NULL;
 	SQLULEN		stmt_timeout;
+	QResultHold	rhold = {0};
 
 	conn = SC_get_conn(self);
 	ci = &(conn->connInfo);
@@ -1943,6 +1973,7 @@ SC_execute(StatementClass *self)
 	if (conn->stmt_timeout_in_effect != stmt_timeout)
 	{
 		char query[64];
+		QResultClass *res;
 
 		SPRINTF_FIXED(query, "SET statement_timeout = %d",
 				 (int) stmt_timeout * 1000);
@@ -1975,11 +2006,13 @@ SC_execute(StatementClass *self)
 	isSelectType = (SC_may_use_cursor(self) || self->statement_type == STMT_TYPE_PROCCALL);
 	if (use_extended_protocol)
 	{
+		QResultClass *first;
+
 		if (issue_begin)
 			CC_begin(conn);
 
-		res = libpq_bind_and_exec(self);
-		if (!res)
+		first = libpq_bind_and_exec(self);
+		if (!first)
 		{
 			if (SC_get_errornumber(self) <= 0)
 			{
@@ -1987,12 +2020,14 @@ SC_execute(StatementClass *self)
 			}
 			goto cleanup;
 		}
+		rhold.first = rhold.last = first;
 	}
 	else if (isSelectType)
 	{
 		char		fetch[128];
 		const char *appendq = NULL;
 		QueryInfo	*qryi = NULL;
+		QResultClass *first;
 
 		qflag |= (SQL_CONCUR_READ_ONLY != self->options.scroll_concurrency ? CREATE_KEYSET : 0);
 		MYLOG(0, "       Sending SELECT statement on stmt=%p, cursor_name='%s' qflag=%d," FORMAT_UINTEGER "\n", self, SC_cursor_name(self), qflag, self->options.scroll_concurrency);
@@ -2010,11 +2045,9 @@ SC_execute(StatementClass *self)
 			appendq = fetch;
 			qflag &= (~READ_ONLY_QUERY); /* must be a SAVEPOINT after DECLARE */
 		}
-		res = SC_get_Result(self);
-		if (self->curr_param_result && res)
-			SC_set_Result(self, res->next);
-		res = CC_send_query_append(conn, self->stmt_with_params, qryi, qflag, SC_get_ancestor(self), appendq);
-		if (useCursor && QR_command_maybe_successful(res))
+		rhold = CC_send_query_append(conn, self->stmt_with_params, qryi, qflag, SC_get_ancestor(self), appendq);
+		first = rhold.first;	
+		if (useCursor && QR_command_maybe_successful(first))
 		{
 			/*
 			 * If we sent a DECLARE CURSOR + FETCH, throw away the result of
@@ -2026,13 +2059,13 @@ SC_execute(StatementClass *self)
 			{
 				QResultClass	*qres, *nres;
 
-				for (qres = res; qres;)
+				for (qres = first; qres;)
 				{
 					if (qres->command && strnicmp(qres->command, "fetch", 5) == 0)
 					{
 						break;
 					}
-					nres = qres->next;
+					nres = QR_nextr(qres);
 					if (nres && QR_get_notice(qres) != NULL)
 					{
 						if (QR_command_successful(nres) &&
@@ -2042,7 +2075,7 @@ SC_execute(StatementClass *self)
 						}
 						QR_add_notice(nres, QR_get_notice(qres));
 					}
-					qres->next = NULL;
+					QR_detach(qres);
 					QR_Destructor(qres);
 					qres = nres;
 
@@ -2053,10 +2086,11 @@ SC_execute(StatementClass *self)
 					if (qres->num_cached_rows < qi.row_size)
 						QR_set_reached_eof(qres);
 				}
-				res = qres;
+				first = qres;
+				rhold.first = first;
 			}
-			if (res && SC_is_with_hold(self))
-				QR_set_withhold(res);
+			if (first && SC_is_with_hold(self))
+				QR_set_withhold(first);
 		}
 		MYLOG(0, "     done sending the query:\n");
 	}
@@ -2064,7 +2098,7 @@ SC_execute(StatementClass *self)
 	{
 		/* not a SELECT statement so don't use a cursor */
 		MYLOG(0, "      it's NOT a select statement: stmt=%p\n", self);
-		res = CC_send_query(conn, self->stmt_with_params, NULL, qflag, SC_get_ancestor(self));
+		rhold = CC_send_query_append(conn, self->stmt_with_params, NULL, qflag, SC_get_ancestor(self), NULL);
 	}
 
 	if (!isSelectType)
@@ -2093,10 +2127,12 @@ SC_execute(StatementClass *self)
 	LEAVE_INNER_CONN_CS(func_cs_count, conn);
 
 	/* Check the status of the result */
-	if (res)
+	if (rhold.first)
 	{
-		was_ok = QR_command_successful(res);
-		was_nonfatal = QR_command_nonfatal(res);
+		QResultClass *first = rhold.first;
+
+		was_ok = QR_command_successful(first);
+		was_nonfatal = QR_command_nonfatal(first);
 
 		if (0 < SC_get_errornumber(self))
 			;
@@ -2113,14 +2149,14 @@ SC_execute(StatementClass *self)
 				SC_set_errornumber(self, STMT_INFO_ONLY);
 		}
 		else
-			SC_set_errorinfo(self, res, 0);
+			SC_set_errorinfo(self, first, 0);
 		/* set cursor before the first tuple in the list */
 		self->currTuple = -1;
 		SC_set_current_col(self, -1);
 		SC_set_rowset_start(self, -1, FALSE);
 
 		/* issue "ABORT" when query aborted */
-		if (QR_get_aborted(res))
+		if (QR_get_aborted(first))
 		{
 		}
 		else
@@ -2128,7 +2164,7 @@ SC_execute(StatementClass *self)
 			QResultClass	*tres;
 
 			/* see if the query did return any result columns */
-			for (tres = res, numcols = 0; !numcols && tres; tres = tres->next)
+			for (tres = first, numcols = 0; !numcols && tres; tres = QR_nextr(tres))
 			{
 				numcols = QR_NumResultCols(tres);
 			}
@@ -2139,13 +2175,13 @@ SC_execute(StatementClass *self)
 				extend_column_bindings(opts, numcols);
 				if (opts->bindings == NULL)
 				{
-					QR_Destructor(res);
+					QR_Destructor(first);
 					SC_set_error(self, STMT_NO_MEMORY_ERROR,"Could not get enough free memory to store the binding information", func);
 					goto cleanup;
 				}
 			}
 
-MYLOG(DETAIL_LOG_LEVEL, "!!%p->miscinfo=%x res=%p\n", self, self->miscinfo, res);
+MYLOG(DETAIL_LOG_LEVEL, "!!%p->miscinfo=%x res=%p\n", self, self->miscinfo, first);
 			/*
 			 * special handling of result for keyset driven cursors.
 			 * Use the columns info of the 1st query and
@@ -2155,16 +2191,14 @@ MYLOG(DETAIL_LOG_LEVEL, "!!%p->miscinfo=%x res=%p\n", self, self->miscinfo, res)
 			    SQL_CONCUR_READ_ONLY != self->options.scroll_concurrency &&
 			    !useCursor)
 			{
-				if (tres = res->next, tres)
+				if (tres = QR_nextr(first), tres)
 				{
-					QR_set_fields(tres, QR_get_fields(res));
-					QR_set_fields(res,  NULL);
-					tres->num_fields = res->num_fields;
-					res->next = NULL;
-					QR_Destructor(res);
-					SC_init_Result(self);
+					QR_set_fields(tres, QR_get_fields(first));
+					QR_set_fields(first,  NULL);
+					tres->num_fields = first->num_fields;
+					QR_detach(first);
 					SC_set_Result(self, tres);
-					res = tres;
+					rhold = self->rhold;
 				}
 			}
 		}
@@ -2192,19 +2226,12 @@ MYLOG(DETAIL_LOG_LEVEL, "!!%p->miscinfo=%x res=%p\n", self, self->miscinfo, res)
 
 	}
 	if (!SC_get_Result(self))
-		SC_set_Result(self, res);
+		SC_set_ResultHold(self, rhold);
 	else
 	{
-		QResultClass	*last;
-
-		for (last = SC_get_Result(self); NULL != last->next; last = last->next)
-		{
-			if (last == res)
-				break;
-		}
-		if (last != res)
-			last->next = res;
-		self->curr_param_result = 1;
+		if (self->rhold.last != rhold.first)
+			QR_concat(self->rhold.last, rhold.first);
+		self->rhold.last = rhold.last;
 	}
 	if (NULL == SC_get_Curres(self))
 		SC_set_Curres(self, SC_get_Result(self));
@@ -2506,10 +2533,7 @@ QResultClass *add_libpq_notice_receiver(StatementClass *stmt, notice_receiver_ar
 {
 	QResultClass *res = NULL, *newres = NULL;
 
-	if (stmt->curr_param_result)
-		for (res = SC_get_Result(stmt); NULL != res && NULL != res->next; res = res->next);
-	if (!res)
-		newres = res = QR_Constructor();
+	newres = res = QR_Constructor();
 	nrarg->conn = SC_get_conn(stmt);
 	nrarg->comment = __FUNCTION__;
 	nrarg->res = res;
@@ -2637,7 +2661,7 @@ libpq_bind_and_exec(StatementClass *stmt)
 	}
 
 	/* 3. Receive results */
-MYLOG(DETAIL_LOG_LEVEL, "get_Result=%p %p %d\n", res, SC_get_Result(stmt), stmt->curr_param_result);
+MYLOG(DETAIL_LOG_LEVEL, "get_Result=%p %p\n", res, SC_get_Result(stmt));
 	pgresstatus = PQresultStatus(pgres);
 	switch (pgresstatus)
 	{
