@@ -205,8 +205,9 @@ struct StatementClass_
 {
 	ConnectionClass *hdbc;		/* pointer to ConnectionClass this
 								 * statement belongs to */
-	QResultClass *result;		/* result of the current statement */
+	QResultHold rhold;
 	QResultClass *curres;		/* the current result in the chain */
+	QResultClass *parsed;		/* parsed result before exec */
 	HSTMT	   *phstmt;
 	StatementOptions options;
 	StatementOptions options_orig;
@@ -277,7 +278,6 @@ struct StatementClass_
 	po_ind_t	lock_CC_for_rb;	/* lock CC for statement rollback ? */
 	po_ind_t	join_info;	/* have joins ? */
 	po_ind_t	parse_method;	/* parse_statement is forced or ? */
-	po_ind_t	curr_param_result; /* current param result is set ? */
 	po_ind_t	has_notice; /* exec result contains notice messages ? */
 	pgNAME		cursor_name;
 	char		*plan_name;
@@ -322,9 +322,12 @@ struct StatementClass_
 #define SC_get_conn(a)	  ((a)->hdbc)
 void SC_init_Result(StatementClass *self);
 void SC_set_Result(StatementClass *self, QResultClass *res);
-#define SC_get_Result(a)  ((a)->result)
+void SC_set_ResultHold(StatementClass *self, QResultHold rhold);
+QResultClass *SC_get_lastres(StatementClass *stmt);
+#define SC_get_Result(a)  ((a)->rhold).first
 #define SC_set_Curres(a, b)  ((a)->curres = b)
 #define SC_get_Curres(a)  ((a)->curres)
+#define SC_get_Parsed(a)  ((a)->parsed ? (a)->parsed : (a)->curres)
 #define SC_get_ARD(a)  ((a)->ard)
 #define SC_get_APD(a)  ((a)->apd)
 #define SC_get_IRD(a)  ((a)->ird)
@@ -447,6 +450,7 @@ enum
 #define SC_set_no_readonly(a)	((a)->execinfo &= ~(1L << 1))
 #define SC_is_readonly(a)	(((a)->execinfo & (1L << 1)) != 0)
 #define SC_execinfo_clear(a)	(((a)->execinfo = 0)
+#define	STMT_HAS_NO_JOIN	0L
 #define	STMT_HAS_OUTER_JOIN	1L
 #define	STMT_HAS_INNER_JOIN	(1L << 1)
 #define SC_has_join(a)		(0 != (a)->join_info)
